/*
** Alias v1.1
**
** A WWW User Redirector
**
** Copyright (c) 28.7.94 by Andreas Ley <ley@rz.uni-karlsruhe.de>
**
** Permission to use, copy, modify, and distribute this software for any
** purpose and without fee is hereby granted, provided that the above
** copyright notice appears in all copies. This software is provided "as is"
** and without any express or implied warranties.
**
** This program has been tested on a HP9000/720 with HP-UX A.08.07
** In this environment, neither lint -u nor gcc -Wall produce any messages.
** If you encounter any errors or need to make any changes to port it
** to another platform, please contact me.
**
** Usage:
**    - Create an aliases file with entries:
**		<username> <url>
**    - Any URL of the form http://slave.host/~username/ can now be replaced
**	by http://www.host/cgi-bin/alias/username/. If you're using CERN
**	httpd, you could create a httpd.conf entry reading:
**	Map	/~*	/cgi-bin/alias/*
**	and use http://www.host/~username/ as a generic URL
**
** Version 1.1 - 25.11.94
**	Added proxy interaction
**
** Version 1.0 - 28.7.94
**	Initial version
*/

char header[]="alias v1.1\n(c) 1994 by Andreas Ley\n";

/* General definitions */

#define	MASTER	"webmaster@rz.uni-karlsruhe.de"
#define	INTERNAL_PROXY

/* File locations */

#define	ALIASES	"/usr/local/etc/httpd/config/alias.conf"

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/param.h>
#include <sys/types.h>
#include "alias_german.h"
/*
#include "alias_english.h"
*/

#define	FALSE	0		/* This is the naked Truth */
#define	TRUE	1		/* and this is the Light */

/* Global variables */

char	master[]=MASTER;


/* Prints header of a HTML-Request */
void show_header(char *subtitle, char *description)
{
	(void)printf("Content-type: text/html\n\n");
	(void)printf("<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML//EN\">\n");
	(void)printf("<HTML>\n");
	(void)printf("<HEAD>\n");
	(void)printf("<TITLE>%s%s%s</TITLE>\n",TITLE,subtitle?" - ":"",subtitle?subtitle:"");
	(void)printf("<LINK REV=\"made\" HREF=\"mailto:%s\">\n",master);
	(void)printf("</HEAD>\n");
	(void)printf("<BODY>\n");
	(void)printf("<H1>%s</H1>\n",subtitle?subtitle:TITLE);
	if (description) {
		(void)printf(description,master);
		(void)printf("<P>\n");
	}
}



/* Prints trailer of a HTML-Request */
void show_trailer()
{
	(void)printf("</BODY>\n");
	(void)printf("</HTML>\n");
}



/* Shows an error page, based on wrong user input */
void show_error(char *error)
{
	show_header(ERROR,error);
	show_trailer();
	exit(0);
}



/* Shows an error page, based on internal misfunction */
void show_fatal(char *error)
{
	show_header(ERROR,error);
	(void)printf(ERROR_FATAL,master);
	(void)printf("<P>\n");
	show_trailer();
	exit(0);
}



/* Returns a Location: header */
void show_location(char *location)
{
	(void)printf("Location: %s\n",location);
	(void)printf("URI: %s\n\n",location);
}



void usage(char *image)
{
	(void)fprintf(stderr,"Usage: %s [-h] [-v]\n",image);
	exit(1);
}


int main(int argc, char *argv[])
{
	int	c;
	char	buffer[1024],*ptr,*nptr;
	char	*username,*path_info,*url;
	FILE	*src;
	int	found;

	while ((c=getopt(argc,argv,"vh?")) != EOF)
		switch ((char)c) {
		case 'v':
			(void)fprintf(stderr,header);
			exit(0);
		case 'h':
			(void)fprintf(stderr,header);
		case '?':
			usage(argv[0]);
		}

	if (!(username=getenv("PATH_INFO")))
		show_fatal(ERROR_MISSING_USERNAME);
	if (*username=='/')
		username++;
	if ((path_info=strchr(username,'/')))
		*path_info++='\0';
	else
		path_info="";

	if ((src=fopen(ALIASES,"r"))) {
		found=FALSE;
		ptr=buffer;
		while(fgets(buffer,sizeof(buffer),src)) {
			if (*buffer!='#') {
				if ((ptr=strchr(buffer,'\n')))
					*ptr='\0';
				for (ptr=buffer;*ptr&&!strchr(" \t",*ptr);ptr++);
				if (*ptr)
					*ptr++='\0';
				while (strchr(" \t",*ptr))
					ptr++;
				if ((found=!strcmp(buffer,username)))
					break;
			}
		}
		fclose(src);
		if (!found) {
			(void)sprintf(buffer,ERROR_UNKNOWN_USER,username,master);
			show_error(buffer);
		}
		if (!*ptr) {
			(void)sprintf(buffer,ERROR_NOURL,username);
			show_fatal(buffer);
		}
#ifdef	INTERNAL_PROXY
		nptr=strstr(ptr,"://");
		if (nptr) {
			url=malloc(strlen(ptr)+strlen(path_info)+22);
			strcpy(url,"/httpd-internal-proxy/");
			strncpy(url+22,ptr,nptr-ptr);
			strcpy(url+22+(nptr-ptr),nptr+2);
		}
		else {
			url=malloc(strlen(ptr)+strlen(path_info)+2);
			strcpy(url,ptr);
		}
#else
		url=malloc(strlen(ptr)+strlen(path_info)+2);
		strcpy(url,ptr);
#endif
		strcat(url,"/");
		strcat(url,path_info);
	}
	else
		show_fatal(ERROR_READ);

	show_location(url);

	return(0);
}
